/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/
package org.logicblaze.soa.grocery.components;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.CachedXPathAPI;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.management.Attribute;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;

public class InventoryViewTransformComponent extends TransformComponentSupport {
    private static final Log log = LogFactory.getLog(InventoryViewTransformComponent.class);

    protected Properties auth = null;
    protected Properties role = null;

    protected String clientName;
    protected String clientRole;

    public boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws MessagingException {

        try {
            log.debug("Performing service-request to sql-query transformation.");
            if (auth == null) {
                throw new IllegalArgumentException("Authentication store has not been initialized. Found: " + auth);
            }
            log.debug("Server Authentication Map: " + auth);

            if (role == null) {
                throw new IllegalArgumentException("Role store has not been initialized. Found: " + role);
            }
            log.debug("Server Role Map: " + role);

            Node document = ((DOMSource)(new SourceTransformer()).toDOMSource(in)).getNode();

            // Return the exception message
            if (isExceptionXml(document)) {
                log.debug("Found an exception message: " + document);
                out.setContent(in.getContent());
                return true;
            }

            Attribute authEntry = getAuthenticationEntry(document);

            clientName = authEntry.getName();

            // Check if valid username and password
            String passEntry = (String)auth.get(clientName);
            if (passEntry == null || !passEntry.equals(authEntry.getValue())) {
                throw new SecurityException("Cannot access InventoryViewService. Caused by: Invalid username and/or password.");
            }

            // Check client role
            clientRole = (String)role.get(clientName);
            if (clientRole == null || (!clientRole.equalsIgnoreCase("limited") && !clientRole.equalsIgnoreCase("full"))) {
                clientRole = "limited";
            }

            Set fields = getFieldsToView(document);
            String condition = getSqlCondition(document);

            Source query = new StringSource("<sql>" + getQueryStatement(fields, condition) + "</sql>");
            out.setContent(query);

            return true;
        } catch (Exception e) {
            log.error(e.getMessage(), e);

            out.setContent(createExceptionXml(e));
            return true;
//            throw new MessagingException(e);
        }
    }

    public Attribute getAuthenticationEntry(Node node) throws TransformerException {
        CachedXPathAPI xpath = new CachedXPathAPI();

        Node authNode = xpath.selectSingleNode(node, "item_request/auth");
        if (authNode == null) {
            throw new IllegalArgumentException("No authentication element found in item request. Expected: <auth> element.");
        }

        Node username = xpath.selectSingleNode(authNode, "@username");
        Node password = xpath.selectSingleNode(authNode, "@password");

        if (username == null || password == null) {
            throw new IllegalArgumentException("No username and/or password entry. Expected <auth username='user' password='pass'/>.");
        }

        log.debug("Client Authentication Entry: username=" + username.getNodeValue() + ", password=" + password.getNodeValue());
        return new Attribute(username.getNodeValue(), password.getNodeValue());
    }

    public Set getFieldsToView(Node node) throws TransformerException {
        CachedXPathAPI xpath = new CachedXPathAPI();

        NodeList fieldList = xpath.selectNodeList(node, "item_request/fields/field/child::text()");

        Set fields = new HashSet(fieldList.getLength());
        for (int i=0; i<fieldList.getLength(); i++) {
            fields.add(fieldList.item(i).getNodeValue());
        }

        log.debug("Client Fields Entry: " + fields);
        return fields;
    }

    public String getSqlCondition(Node node) throws TransformerException {
        CachedXPathAPI xpath = new CachedXPathAPI();

        NodeList condList = xpath.selectNodeList(node, "item_request/condition/child::text()");
        if (condList==null || condList.getLength() < 1) {
            return "";
        }

        StringBuffer cond = new StringBuffer("(" + condList.item(0).getNodeValue() + ")");

        for (int i=1; i<condList.getLength(); i++) {
            cond.append(" AND (" + condList.item(i).getNodeValue() + ")");
        }

        log.debug("Client Condition Entry: " + cond.toString());
        return cond.toString();
    }

    public String getQueryStatement(Set fields, String condition) {
        StringBuffer query = new StringBuffer();

        query.append("SELECT ");

        // Field List
        if (fields.size() > 0) {
            Iterator i = fields.iterator();
            query.append(i.next());
            while (i.hasNext()) {
                query.append("," + i.next());
            }
        } else {
            query.append("*");
        }

        // From table
        query.append(" FROM items");

        // Limit depending on the client role
        if (clientRole.equalsIgnoreCase("limited")) {
            // Limited role only
            query.append(" WHERE (supplied_by = '" + clientName + "')");

            if (condition != null && condition.length() > 0) {
                query.append(" AND (" + condition + ")");
            }

        } else {
            // Full access
            if (condition != null && condition.length() > 0) {
                query.append(" WHERE " + condition);
            }
        }

        log.debug("SQL Entry: " + query.toString());
        return query.toString();
    }

    public Properties getAuth() {
        return auth;
    }

    public void setAuth(Properties auth) {
        this.auth = auth;
    }

    public Properties getRole() {
        return role;
    }

    public void setRole(Properties role) {
        this.role = role;
    }


    // Custom method to create and handle exception
    public boolean isExceptionXml(Node document) {
        return document.getFirstChild().getNodeName().equalsIgnoreCase("exception");
    }

    public Source createExceptionXml(Throwable e) {
        return new StringSource("<exception class='" + e.getClass() + "'>'" + e.getMessage() + "'</exception>");
    }
}
